<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>{{title}}</title>
    <style type="text/css">
        {% include "styles.css" %}
    </style>
</head>
<body>
    {% block content %}
    {% endblock %}
<p class="footer">
    Generated by junit2html
</p>
</body>
</html>