h1 {
    font-family: sans-serif;
}
h2 {
    font-family: sans-serif;
}

li > a {
    font-family: monospace;
}

body {
    margin-bottom: 3em;
}

.testclass > td {
    border-bottom: 1px solid silver;
    color: #222222;
    padding: 3px;
    font-weight: bold;
}

.testclass > div {
    padding-left: 1em;
}

.testclass > a > div {
    font-size: 1.3em;
    font-family: sans-serif;
}

.testcase > div {
    border-left: 3px solid black;
    padding-left: 1em;
}

.testcase pre {
    display: block;
    padding: 4px;
    margin-right: 1em;
    background-color: #121212;
    color: #dedede;
    white-space: pre-wrap;
}

.testcase .details b {
    font-size: 1.2em;
}

.stdout, .stderr, .property {
    margin-left: 1em;
}

.testcase-link {
    text-decoration: none;
}

.testcase-cell {
    height: 1.8em;
    width: 1.8em;
    vertical-align: middle;
    text-align: center;
}
.testcase-cell a {
    display: inline-block;
    margin: -1em;
    padding: 1em;
}

.testcase-combined {
    white-space: pre;
    padding-left: 1em;
}

.skipped {
    background-color: gold;
}

.failed {
    background-color: lightcoral;
}

.passed {
    background-color: limegreen;
}

.tooltip {
    visibility: hidden;
    padding: 2px;
    z-index: 1;
    position: absolute;
    background: cornsilk;
    border: 1px solid black;
    margin-top: 15px;
    margin-left: 10px;
    opacity: 0;
}
.tooltip-parent:hover .tooltip {
    visibility: visible;
    opacity: 1;
}

.testcase:hover {
    background-color: gray;
    color: white;
}

.result-stats {
    margin: auto;
}
