body {
    background-color: #111111;
    color: silver;
    padding-bottom: 20em;
    margin: 0;
    min-height: 15cm;
}

a {
    color: #007acc;
}

h1, h2, h3, h4, h5, h6, h7 {
    font-family: sans-serif;
}

h1 {
    color: cadetblue;
    padding: 3mm;
    margin-top: 0;
    margin-bottom: 1mm;
}

.footer {
    font-style: italic;
    font-size: small;
    text-align: right;
    padding: 1em;
}

.testsuite {
    padding-bottom: 2em;
    margin-left: 1em;
}

.proplist {
    width: 100%;
    margin-bottom: 2em;
    border-collapse: collapse;
    border: 1px solid grey;
}

.proplist th {
    background-color: darkslategrey;
    width: 5em;
    padding: 2px;
    padding-right: 1em;
    text-align: left;
    font-family: monospace;
}

.proplist td {
    padding: 2px;
    word-break: break-all;
}

.index-table {
    width: 90%;
    margin-left: 1em;
}

.index-table td {
    vertical-align: top;
    max-width: 200px;
    word-wrap: break-word;
}

.failure-index {

}

.toc {
    margin-bottom: 2em;
    font-family: monospace;
}

.stdio, pre {
    min-height: 1em;
    background-color: #1e1e1e;
    color: silver;
    padding: 0.5em;
}
.tdpre {
    background-color: #1e1e1e;
}

.test {
    margin-left: 0.5cm;
}

.toc {
    list-style: none;
}

.toc li.outcome {
    list-style: none;
}

.toc li.outcome::before {
    content: "\2022";
    color: black;
    font-weight: bold;
    display: inline-block;
    width: 1.27em;
    margin-left: -1.27em;
    font-size: 1.2em;
}

.toc li.outcome-failed {
    background-color: #440000;
}

.toc li.outcome-failed::before {
    color: red;
}

.toc li.outcome-passed::before {
    color: lightseagreen;
}

.toc li.outcome-skipped::before {
    color: orange;
}

.testcases .outcome {
    border-left: 1em;
    padding: 2px;
}

.testcases .outcome-failed {
    border-left: 1em solid palevioletred;
}


.outcome-passed {
    border-left: 1em solid seagreen;
}

.outcome-skipped {
    border-left: 1em dotted silver;
}

.testcases .outcome-passed {
    border-left: 1em solid seagreen;
}

.testcases .outcome-skipped {
    border-left: 1em solid grey;
}

.stats-table {
}

.stats-table td {
    min-width: 4em;
    text-align: right;
}

.stats-table .failed {
    background-color: darkred;
}

.stats-table .passed {
    background-color: seagreen;
}

.matrix-table {
    table-layout: fixed;
    border-spacing: 0;
    width: available;
    margin-left: 1em;
}

.matrix-table td {
    vertical-align: center;
}

.matrix-table td:last-child {
    width: 0;
}

.matrix-table tr:hover {
    background-color: darkgoldenrod;
}

.matrix-axis-name {
    white-space: nowrap;
    padding-right: 0.5em;
    border-left: 1px solid black;
    border-top: 1px solid black;
    text-align: right;
}

.matrix-axis-line {
    border-left: 1px solid black;
    width: 0.5em;
}

.matrix-classname {
    text-align: left;
    width: 100%;
    border-top: 2px solid grey;
    border-bottom: 1px solid silver;
}

.matrix-casename {
    text-align: left;
    font-weight: normal;
    font-style: italic;
    padding-left: 1em;
    border-bottom: 1px solid silver;
}

.matrix-result {
    display: block;
    width: 1em;
    text-align: center;
    padding: 1mm;
    margin: 0;
}

.matrix-result-combined {
    white-space: nowrap;
    padding-right: 0.2em;
    text-align: right;
}

.matrix-result-failed {
    background-color: palevioletred;
}

.matrix-result-passed {
    background-color: seagreen;
}

.matrix-result-skipped {
    background-color: darkgoldenrod;
}

.matrix-even {
    background-color: dimgrey;
}
